<?php

namespace Drupal\simplenews_stats;

use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Entity\EntityInterface;
use Drupal\simplenews\SubscriberInterface;

/**
 * 
 */
class SimplenewsStatsEngine {

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * Current route match
   * @var Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $current_route_match;

  /**
   * Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entity_type_manager;

  public function __construct(RequestStack $request, CurrentRouteMatch $current_route_match, EntityTypeManager $entity_type_manager) {
    $this->request             = $request;
    $this->current_route_match = $current_route_match;
    $this->entity_type_manager = $entity_type_manager;
  }

  /**
   * 
   * @param string $tag
   */
  public function AddStatTags($tag, $path = null) {

    $entities = $this->getTagEntities($tag);
    if ($entities === FALSE)
      return;

    if (empty($path))
      $path = $this->request->getCurrentRequest()->getPathInfo();

    $this->logHit($entities['subscriber'], $entities['entity'], $this->current_route_match->getRouteName(), $path);
  }

  /**
   * Return entities associated to the tag.
   * 
   * @param string $tag The tag.
   * @return bool|array Array of entities or false.
   */
  public function getTagEntities($tag) {

    if (!preg_match('/^u[0-9]*nl[0-9]*/', $tag))
      return FALSE;

    $args = preg_split("/(u)|(nl)/", $tag);

    if (count($args) != 3)
      return FALSE;

    $subscriber = $this->entity_type_manager->getStorage('simplenews_subscriber')->load($args[1]);
    $entity     = $this->entity_type_manager->getStorage('node')->load($args[2]);

    if ($subscriber == FALSE || $entity == FALSE)
      return FALSE;

    return [
      'subscriber' => $subscriber,
      'entity'     => $entity
    ];
  }

  /**
   * Store in data base the newsletter hit (click or view)
   * @param SubscriberInterface $subscriber
   * @param EntityInterface $entity
   * @param string $route_name
   * @param string $path
   */
  protected function logHit(SubscriberInterface $subscriber, $entity, $route_name, $path) {
    if ($route_name === 'simplenews_stats.hit_view')
      $action = 'view';
    else
      $action = 'click';

    $data = [
      'uid'         => $subscriber->getUserId(),
      'snid'        => $subscriber->id(),
      'email'       => $subscriber->getMail(),
      'title'       => $action,
      'entity_type' => $entity->getEntityTypeId(),
      'entity_id'   => $entity->id(),
      'route_path'  => substr($path, 0, 255),
      'created'     => \Drupal::time()->getRequestTime()
    ];

    $this->globalStatUpdate($subscriber, $entity, $action);

    $storage         = $this->entity_type_manager->getStorage('simplenews_stats_item');
    $simplenews_stat = $storage->create($data);
    $simplenews_stat->save();
  }

  /**
   * 
   */
  public function logHitSent(SubscriberInterface $subscriber, $entity) {
    $simplenews_stats = $this->getSimplenewsStats($subscriber, $entity);
    $simplenews_stats->increaseTotalMail()
        ->save();
  }

  /**
   * Update the global stat entry
   * @param SubscriberInterface $subscriber
   * @param \Drupal\Core\Entity\EntityInterface $entity
   */
  protected function globalStatUpdate(SubscriberInterface $subscriber, EntityInterface $entity, $action) {
    $storage   = $this->entity_type_manager->getStorage('simplenews_stats');
    $entity_gs = $storage->getFromRelatedEntity($entity);

    if (!$entity_gs) {
      $entity_gs = $storage->createFromSubscriberAndEntity($subscriber, $entity);
    }

    $entity_gs->{'increase' . ucfirst($action)}()
        ->save();
  }

  /**
   * Return the simplenews stats entitys
   * @return SubscriberInterface Description
   */
  protected function getSimplenewsStats(SubscriberInterface $subscriber, $entity) {
    $storage   = $this->entity_type_manager->getStorage('simplenews_stats');
    $entity_gs = $storage->getFromRelatedEntity($entity);

    if ($entity_gs)
      return $entity_gs;

    return $storage->createFromSubscriberAndEntity($subscriber, $entity);
  }

}
